import http from '../utils/http'
import { getToken } from '../utils/auth'
import { isArray } from 'lodash'


/**
 * 
 * @param {*} username 
 * @param {*} password 
 */
export const login = (username, password) => async dispatch => {
    dispatch({ type: 'LOGIN_REQUEST' })

    try {
        const response = await http.post('auth/get_token/', { username, password })

        if (response.status === 200) {
            dispatch({ type: 'LOGIN_SUCCESS', payload: response.data })    
        }

        if (response.status === 401) {
            dispatch({ type: 'LOGIN_FAILURE' })    
        }
    } catch(error) {
        dispatch({ type: 'LOGIN_FAILURE' })
    }
}

/**
 * 
 */
export const checkToken = () => async dispatch => {
    dispatch({ type: 'CHECK_TOKEN_REQUEST' })

    try {
        const response = await http.post('auth/check_token/', { token: getToken() })

        if (response.status === 200) {
            dispatch({ type: 'CHECK_TOKEN_SUCCESS', payload: response.data })
        }

        if (response.status === 401) {
            dispatch({ type: 'CHECK_TOKEN_FAILURE' })
        }
    } catch (error) {
        dispatch({ type: 'CHECK_TOKEN_FAILURE' })
    }
}

/**
 * 
 */
export const logout = () => {
    return {
        type: 'LOGOUT_REQUEST'
    }
}

/**
 * 
 * @param {*} message 
 */
export const spinner = (show, message) => dispatch => {
    dispatch({
        type: show ? 'SHOW_SPINNER' : 'HIDE_SPINNER',
        payload: message
    })
}

/**
 * 
 * @param {*} text 
 */
export const notify = message => (dispatch, getState)  => {
    dispatch({
        type: message ? 'SHOW_NOTIFICATION' : 'HIDE_NOTIFICATION',
        payload: { message }
    })
}

/**
 * 
 * @param {*} resource 
 */
export const get = resource => async (dispatch, getState) => {
    dispatch({ type: 'REQUEST_INIT' })

    try {
        const response = await http.get(`${resource}`)
        return dispatch({ type: 'REQUEST_OK', payload: response.data })
    } catch (error) {
        return dispatch({ type: 'REQUEST_KO', payload: error.response.status })
    }
}

/**
 * 
 * @param {*} resource 
 */
export const post = (resources, data) => async dispatch => {
    dispatch({ type: 'REQUEST_INIT' })
    
    data = data || {}

    if (isArray(resources)) {
        for (let r of resources) {
            try {
                const response = await http.post(`${r}`, data)
                dispatch({ type: 'REQUEST_OK', payload: response.data })
            } catch (error) {
                dispatch({ type: 'REQUEST_KO', payload: error.response.status })
            }
        }
    } else {
        try {
            const response = await http.post(`${resources}`, data)
            dispatch({ type: 'REQUEST_OK', payload: response.data })
        } catch (error) {
            dispatch({ type: 'REQUEST_KO', payload: error.response.status })
        }
    }
}
