import React, { Component } from 'react'
import { connect } from 'react-redux'
import AppBar from 'material-ui/AppBar'
import Toolbar from 'material-ui/Toolbar'
import Typography from 'material-ui/Typography'
import Avatar from 'material-ui/Avatar'
import Button from 'material-ui/Button'
import Menu, { MenuItem } from 'material-ui/Menu'
import UserIcon from '../icons/UserIcon'
import { logout } from '../../actions'
import { withStyles } from 'material-ui/styles'

const styles = theme => ({
    root: {
        zIndex: theme.zIndex.drawer + 1,
    },
    titleBar: {
        flexGrow: 1
    },
    userBar: {
        display: 'flex',
    },
    avatar: {
        color: '#fff',
        backgroundColor: '#fff'
    }
})

class Topbar extends Component {
    constructor(props) {
        super(props)

        this.state = {
            anchorEl: null
        }
    }

    /**
     * 
     */
    handleOpenMenu = e => {
        this.setState({
            anchorEl: e.currentTarget
        })
    }

    /**
     * 
     */
    handleMenuClose = e => {
        this.setState({
            anchorEl: null
        })
    }

    /**
     * 
     */
    handleSessionClose = e => {
        this.setState({
            anchorEl: null
        })

        this.props.closeSession()
    }

    /**
     * 
     */
    render() {
        const { username, classes } = this.props
        const { anchorEl } = this.state

        return (
            <AppBar position="absolute" className={classes.root}>
                <Toolbar>
                    <Typography className={classes.titleBar} variant="title" color="inherit" noWrap>
                        Eiru Automation
                    </Typography>
                    <div className={classes.userBar}>
                        <Avatar className={classes.avatar}>
                            <UserIcon />
                        </Avatar>
                        <Button 
                            aria-owns={anchorEl ? "menu-appbar" : null} 
                            aria-haspopup="true" 
                            color="inherit" 
                            onClick={this.handleOpenMenu}
                        >
                            {username}
                        </Button>
                        <Menu
                            id="menu-appbar"
                            open={Boolean(anchorEl)} 
                            anchorEl={anchorEl}
                            anchorOrigin={{
                                vertical: "top",
                                horizontal: "right",
                            }}
                            transformOrigin={{
                                vertical: "top",
                                horizontal: "right",
                            }}
                            onClose={this.handleMenuClose}
                        >
                            <MenuItem onClick={this.handleSessionClose}>Cerrar sesión</MenuItem>
                        </Menu>
                    </div>
                </Toolbar>
            </AppBar>
        )
    }
}

/**
 * 
 * @param {*} state 
 * @param {*} props 
 */
const mapStateToProps = (state, props) => {
    return {
        username: state.auth.username
    }
}

/**
 * 
 * @param {*} dispatch 
 * @param {*} props 
 */
const mapDispatchToProps = (dispatch, props) => ({
    closeSession() {
        dispatch(logout())
    }
})

Topbar = withStyles(styles)(Topbar)

export default connect(mapStateToProps, mapDispatchToProps)(Topbar)
