import { createReducer } from '../utils/reducer'
import { setToken } from '../utils/auth'

const initialState = {
    isTokenCheck: false,
    isLogging: false,
    isAuthenticated: false,
    token: null,
    username: null
}

/**
 * 
 * @param {*} isTokenCheck 
 * @param {*} action 
 */
const isTokenCheck = (isTokenCheck, action) => {
    if (action.type === 'CHECK_TOKEN_REQUEST') {
        return true
    }

    if (action.type === 'CHECK_TOKEN_SUCCESS') {
        return false
    }

    if (action.type === 'CHECK_TOKEN_FAILURE') {
        setToken(null)
        return false
    }

    return isTokenCheck
}

/**
 * 
 */
const tokenCheckReducer = createReducer(false , {
    'CHECK_TOKEN_REQUEST': isTokenCheck,
    'CHECK_TOKEN_SUCCESS': isTokenCheck,
    'CHECK_TOKEN_FAILURE': isTokenCheck
})

/**
 * 
 * @param {*} isLogging 
 * @param {*} action 
 */
const isLogging = (isLogging, action) => {
    if (action.type === 'LOGIN_REQUEST') {
        return true
    }

    if (action.type === 'LOGIN_SUCCESS' || action.type === 'LOGIN_FAILURE') {
        return false
    }

    return isLogging
}

/**
 * 
 */
const logginReducer = createReducer(false, {
    'LOGIN_REQUEST': isLogging,
    'LOGIN_SUCCESS': isLogging,
    'LOGIN_FAILURE': isLogging
})

/**
 * 
 * @param {*} isAuthenticated 
 * @param {*} action 
 */
const isAuthenticated = (isAuthenticated, action) => {
    if (action.type === 'LOGIN_SUCCESS' || action.type === 'CHECK_TOKEN_SUCCESS') {
        setToken(action.payload.token)
        return true
    }

    if (action.type === 'LOGIN_FAILURE' || action.type === 'CHECK_TOKEN_FAILURE' || action.type === 'LOGOUT_REQUEST') {
        setToken(null)
        return false
    }

    return isAuthenticated
}

/**
 * 
 */
const authenticationReducer = createReducer(false, {
    'LOGIN_SUCCESS': isAuthenticated,
    'LOGIN_FAILURE': isAuthenticated,
    'CHECK_TOKEN_SUCCESS': isAuthenticated,
    'CHECK_TOKEN_FAILURE': isAuthenticated,
    'LOGOUT_REQUEST': isAuthenticated
})

/**
 * 
 * @param {*} username 
 * @param {*} action 
 */
const setUsername = (username, action) => {
    return action.payload.username
}

const userReducer = createReducer(null, {
    'LOGIN_SUCCESS': setUsername,
    'CHECK_TOKEN_SUCCESS': setUsername
})

export const auth = (state = initialState, action) => {
    return {
        isTokenCheck: tokenCheckReducer(state.isTokenCheck, action),
        isLogging: logginReducer(state.isLogging, action),
        isAuthenticated: authenticationReducer(state.isAuthenticated, action),
        username: userReducer(state.username, action)
    }
}