import { createStore, combineReducers, applyMiddleware } from 'redux'
import { routerReducer, routerMiddleware } from 'react-router-redux'
import createBrowserHistory from 'history/createBrowserHistory'
import ThunkMiddleware from  'redux-thunk'
import LoggerMiddleware from 'redux-logger'

import { auth } from './auth'
import { spinner } from './spinner'
import { notification } from './notification'
import { containers } from './containers'
import { task } from './task'
import { requests } from './requests'
import { repositories } from './repositories'

const history = createBrowserHistory()

const rootReducer = combineReducers({
    auth,
    spinner,
    notification,
    containers,
    task,
    requests,
    repositories,
    router: routerReducer
})

const RouterMiddleware = routerMiddleware(history)

let middlewares = [RouterMiddleware, ThunkMiddleware]

if (process.env.NODE_ENV === 'development') {
    middlewares = [...middlewares, LoggerMiddleware]
}

const store = createStore(rootReducer, applyMiddleware(...middlewares))

export {
    history,
    store
}